/* This software comprises contributed code made by Motorola, as a 
 * Contributor, to Network Simulator NS-2 software provided by the 
 * Regents of the University of California.
 * (Copyright; Regents of the University of California, 1994)
 * The contributed code was made as a result of a partnership between 
 * Motorola and INRIA Rhone-Alpes. 
 *
 * Copyright in the contributed code belongs to Motorola Inc. 2001
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ALL ADVERTISING MATERIALS MENTIONING FEATURES OR USE OF THIS SOFTWARE MUST 
 * DISPLAY AN ACKNOWLEDGEMENT TO THE COPYRIGHT OWNERS. 
 * ANY REDISTRIBUTION OF THIS SOFTWARE MUST CONTAIN THE ABOVE COPYRIGHT NOTICES, 
 * CONDITIONS AND DISCLAIMER.
 */

/* ############################################################################
 * This code was developed by Thierry Ernst (1998-2001)
 * MOTOROLA Labs Paris FRANCE - INRIA Rhone-Alpes Grenoble (PLANETE) FRANCE 
 * NS-2.1b6 enhancements for Wide-Area mobility simulations
 * ############################################################################
 */

#include <malloc.h>
#include <stdio.h>
#include <assert.h>
#include <string.h>

#include "addresses.h"

/* ****************************************************************
   address struct 
   **************************************************************** */

struct address {
  int domain ;
  int cluster ;
  int node ;
} ;

address
new_address (int domain, int cluster, int node) {
  address Add ;

  Add = (address) malloc (sizeof (struct address)) ;
  Add->domain = domain ;
  Add->cluster = cluster ;
  Add->node = node ;

  return Add ;
}

void
delete_adddress (address A) {
  free (A) ;
}

char *
address2string (address Add) {
  char * string = (char *) malloc (sizeof (char) * MAX_ADDRESS_STRING_SIZE) ;
  int nb_char ;

  nb_char = snprintf (string, MAX_ADDRESS_STRING_SIZE, "%d.%d.%d", 
		      Add->domain, Add->cluster, Add->node) ;
  assert (nb_char != -1) ;
  
  return (string) ;
}

/* ****************************************************************
   addr_domain struct
   **************************************************************** */

struct addr_domain {
  int nb_domain ;
  int clusters_per_domain[MAX_DOMAINS] ;
  int nodes_per_cluster[MAX_DOMAINS][MAX_CLUSTER_PER_DOMAIN] ;
} ;

addr_domain
new_addr_domain () {
  addr_domain Domain ;

  Domain = (addr_domain) malloc (sizeof (struct addr_domain)) ;
  Domain->nb_domain = 0 ;
 
  return Domain ; 
}

void
delete_addr_domain (addr_domain A) {
  free (A) ;
}
 
int
new_domain (addr_domain Addr_domain) {
  assert (Addr_domain->nb_domain < (MAX_DOMAINS - 1)) ;

  Addr_domain->clusters_per_domain[Addr_domain->nb_domain] = 0 ;
  Addr_domain->nb_domain ++ ;

  return (Addr_domain->nb_domain - 1) ;
}

int
new_cluster (addr_domain Addr_domain, int Domain) {
  int free_cluster ;

  assert (Domain < MAX_DOMAINS) ;
  assert (Domain < Addr_domain->nb_domain) ;
  assert (Addr_domain->clusters_per_domain[Domain] < (MAX_CLUSTER_PER_DOMAIN - 1)) ;

  free_cluster = Addr_domain->clusters_per_domain[Domain] ; 
  Addr_domain->nodes_per_cluster[Domain][free_cluster] = 0 ;
  Addr_domain->clusters_per_domain[Domain] ++ ;

  return free_cluster ;
}

int
new_node (addr_domain Addr_domain, int Domain, int Cluster) {
  int node ;
  
  assert (Domain < MAX_DOMAINS) ;
  assert (Domain < Addr_domain->nb_domain) ;
  assert (Cluster < MAX_CLUSTER_PER_DOMAIN) ;
//  assert (Cluster <= Addr_domain->clusters_per_domain[Domain]) ;
  assert (Addr_domain->nodes_per_cluster[Domain][Cluster] < 
	  (MAX_NODE_PER_CLUSTER - 1)) ;
  
  node = Addr_domain->nodes_per_cluster[Domain][Cluster] ;
  Addr_domain->nodes_per_cluster[Domain][Cluster] ++ ;

  return node ;
}

int
nb_domains (addr_domain A) {
  return A->nb_domain ;
}

int
nb_clusters (addr_domain A, int Domain) {
  assert (Domain < MAX_DOMAINS) ;

  return A->clusters_per_domain[Domain] ;
}

int
nb_nodes (addr_domain A, int Domain, int Cluster) {
  assert (Domain < MAX_DOMAINS) ;
  assert (Domain < A->nb_domain) ;
  assert (Cluster < MAX_CLUSTER_PER_DOMAIN) ;
  assert (Cluster < A->clusters_per_domain[Domain]) ;

  return A->nodes_per_cluster[Domain][Cluster] ;
}

char *
get_addr_domain (addr_domain A) {
  int i, j ;
  char * string = (char *) malloc (sizeof (char) * MAX_ADDR_DOMAIN_STRING_SIZE) ;
  int size, verif ;

  verif = snprintf (string, MAX_ADDR_DOMAIN_STRING_SIZE, "%d ", A->nb_domain) ;
  assert (verif != -1) ;
  size = verif ;

  verif = snprintf (string + size, MAX_ADDR_DOMAIN_STRING_SIZE - size, "{ ") ;
  size += verif ;

  for (i = 0 ; i < A->nb_domain ; i++) {
    verif = snprintf (string + size , MAX_ADDR_DOMAIN_STRING_SIZE - size, "%d ", 
		      A->clusters_per_domain[i]) ;  
    assert (verif != -1) ;
    size += verif ;
  }
  verif = snprintf (string + size, MAX_ADDR_DOMAIN_STRING_SIZE - size, "} { ") ;
  assert (verif != -1) ;
  size += verif ;

  for (i = 0 ; i < A->nb_domain ; i++) 
    for (j = 0 ; j < A->clusters_per_domain[i] ; j++) {
      verif = snprintf (string + size , MAX_ADDR_DOMAIN_STRING_SIZE - size, 
		     "%d ", A->nodes_per_cluster[i][j]) ;
      size += verif ;
    }
  verif = snprintf (string + size, MAX_ADDR_DOMAIN_STRING_SIZE - size,"}") ;
  assert (verif != -1) ;
  
  return string ;
}
