/* This software comprises contributed code made by Motorola, as a
 * Contributor, to Network Simulator NS-2 software provided by the
 * Regents of the University of California.
 * (Copyright; Regents of the University of California, 1994)
 * The contributed code was made as a result of a partnership between
 * Motorola and INRIA Rhone-Alpes.
 *
 * Copyright in the contributed code belongs to Motorola Inc. 2001
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ALL ADVERTISING MATERIALS MENTIONING FEATURES OR USE OF THIS SOFTWARE MUST
 * DISPLAY AN ACKNOWLEDGEMENT TO THE COPYRIGHT OWNERS.
 * ANY REDISTRIBUTION OF THIS SOFTWARE MUST CONTAIN THE ABOVE COPYRIGHT NOTICES,
 * CONDITIONS AND DISCLAIMER.
 */

/* ############################################################################
 * This code was developed by Mickael Vera (1999)
 * MOTOROLA Labs Paris FRANCE
 * NS-2.1b6 enhancements for Wide-Area mobility simulations
 * ############################################################################
 */



#include <time.h>
#include <math.h>
#include <malloc.h>
#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

// Begin Thierry 02/2000
// Vera used it for debugging only - does not work on Solaris

// #include <nan.h>
#define NAN	-1
// End Thierry


#include "points_distribution.h"



#define SQR(a) (a * a)

point *
points_distribution (int nb_points, int X, int Y, float pertubation) 
{
  static int rand_test = 0 ;
#ifdef ABSORBTION
  FILE * absorbtion_plot = NULL ;
#endif
  int i = 0, j = 0, k = 0 ;
  point * points_array = NULL ; /* meters */
  point points_speeds[nb_points] ; /* meters / second */
  int finish = 0 ; 
  point offset[9] ;
  point vector ;
  point mirror_point ;
  double distance = -1 ;
  int loops = 0 ;
  
  vector.x = NAN ;
  vector.y = NAN ;

#ifdef ABSORBTION
  absorbtion_plot = fopen ("absorbtion_plot_data", "w") ;
#endif

  assert (WINDY_COEFF * DT / MASS < 1) ;
  
  /* 
     Here is the positions of the squares
     1 2 3
     8 0 4
     7 6 5
  */
  offset[0].x = 0 ;
  offset[0].y = 0 ;
  offset[1].x = -1 ;
  offset[1].y = -1 ;
  offset[2].x = 0 ;
  offset[2].y = -1;
  offset[3].x = 1 ;
  offset[3].y = -1;
  offset[4].x = 1 ;
  offset[4].y = 0 ;
  offset[5].x = 1 ;
  offset[5].y = 1 ;
  offset[6].x = 0 ;
  offset[6].y = 1 ;
  offset[7].x = -1;
  offset[7].y = 1 ;
  offset[8].x = -1;
  offset[8].y = 0 ;

 
  points_array = malloc (nb_points * sizeof (struct point)) ;
  
  if (rand_test == 0) {
    /*  srand ((int) time(NULL)) ; */
    srand (7) ;
    rand_test = 1 ;
  }
  
  for (i = 0; i < nb_points; i++){
    points_array[i].x = (int) ((((float) X) * rand ()) / (RAND_MAX + 1.0)) ; 
    points_array[i].y = (int) ((((float) Y) * rand ()) / (RAND_MAX + 1.0)) ;
    points_speeds[i].x = 0 ;
    points_speeds[i].y = 0 ;
  }

  while ((! finish) && (loops < 20001)) {
    /* compute wind acceleration */
    for (i = 0; i < nb_points; i++) {
      points_speeds[i].x = points_speeds[i].x * (1 - WINDY_COEFF * DT / MASS) ;
      points_speeds[i].y = points_speeds[i].y * (1 - WINDY_COEFF * DT / MASS) ;
    }
    
    /* compute 1/r^2 accelerations */
    for (i = 0; i < nb_points; i++) { /* point on which strong applies */
      for (j = 0 ; j < nb_points; j++) { /* point that exercess strong */
	for (k = 0; k < 9; k++) { /* square in which is the point */
	  if (! (k == 0 && i == j)){
	    /* compute unit vector from real point to virtual points */
	    do {
	      compute_mirror_point (&mirror_point, j, points_array, k, offset, X, Y) ;
	      vector.x = mirror_point.x - points_array[i].x ;
	      vector.y = mirror_point.y - points_array[i].y ;
	      distance = sqrt (SQR(vector.x) + SQR(vector.y)) ;
	      if (distance == 0) {
		/* change this line, it should depend on the size of the area 
		   and on the number of nodes */  
		if (points_array[i].x > 1) /* avoid leaving the area */
		  points_array[i].x = points_array[i].x - 0.1 ;
		else 
		  points_array[i].x = points_array[i].x + 0.1 ;
		if (points_array[i].y > 1) /* avoid leaving the area */
		  points_array[i].y = points_array[i].y - 0.1 ;
		else 
		  points_array[i].y = points_array[i].y + 0.1 ;
	      }
	    } while (distance == 0) ;
  	    vector.x = vector.x / distance ;
	    vector.y = vector.y / distance ;
	    /* update speeds */
	    speeds_increase (points_speeds, &vector, i, j, distance) ;
	  }
	}
      }
    }
    for (i = 0; i < nb_points; i++) {
      update_position_and_speeds (points_array, i, points_speeds, X, Y) ;
    }
#ifdef VERBOSE
    if (loops % 100 == 0){
      fprintf (stdout, ".") ;
      fflush (stdout) ;
    }
#endif
    loops ++ ;
    finish = (immobile_points (points_speeds, nb_points) && loops > 10) ;
#ifdef ABSORBTION
    fprintf (absorbtion_plot, "%d %f\n", loops, points_speeds[0].y) ;
#endif


  }
#ifdef VERBOSE
  fprintf (stdout, "  %d loops\n", loops) ;
#endif
#ifdef ABSORBTION
  fclose (absorbtion_plot);
  system ("gnuplot absorbtion_plot_cmd");
#endif
  return (points_array) ;
}

void
speeds_increase (point * speed, point * unit_vector, int i, int j, 
		 double distance) 
{
  speed[i].x = speed[i].x - (unit_vector->x / SQR(distance)) * STRENGTH_COEFF * DT / MASS ;
  speed[i].y = speed[i].y - (unit_vector->y / SQR(distance)) * STRENGTH_COEFF * DT / MASS ;
}

void
update_position_and_speeds (point * points, int i, point * speeds,
			    int X, int Y)
{
  point new_point ;
  
  assert (points[i].x > 0) ;
  assert (points[i].y > 0) ;
  assert (points[i].x < X) ;
  assert (points[i].y < Y) ;
  
  if (speeds[i].x > X/(10 * DT)){
    speeds[i].x = X/(10 * DT) ;
#ifdef WARNINGS
    fprintf (stderr, "Speed overflow\n") ;
#endif
  }
  if (speeds[i].x < - X/(10 * DT)){
    speeds[i].x = - X/(10 * DT) ;
#ifdef WARNINGS
    fprintf (stderr, "Speed overflow\n") ;
#endif
  }
  if (speeds[i].y > Y/(10 * DT)){
    speeds[i].y = Y/(10 * DT) ;
#ifdef WARNINGS
    fprintf (stderr, "Speed overflow\n") ;
#endif
  }
  if (speeds[i].y < - Y/(10 * DT)){
    speeds[i].y = - Y/(10 * DT) ;
#ifdef WARNINGS
    fprintf (stderr, "Speed overflow\n") ;
#endif
  }

  new_point.x = DT * speeds[i].x + points[i].x ;
  new_point.y = DT * speeds[i].y + points[i].y ;
  
  if (new_point.x < 0) {
    new_point.x = - new_point.x ;
    speeds[i].x = - speeds[i].x ;
  }
  
  if (new_point.y < 0) {
    new_point.y = - new_point.y ;
    speeds[i].y = - speeds[i].y ;
  }
  
  if (new_point.x > X) {
    new_point.x = 2 * X - new_point.x ;
    speeds[i].x = - speeds[i].x ;
  }
    
  if (new_point.y > Y) {
    new_point.y = 2 * Y - new_point.y ;
    speeds[i].y = - speeds[i].y ;
  }

  points[i].x = new_point.x ;
  points[i].y = new_point.y ;

  assert (points[i].x > 0) ;
  assert (points[i].y > 0) ;
  assert (points[i].x < X) ;
  assert (points[i].y < Y) ;
}

int
immobile_points (point * speeds, int nb_points) 
{
  float speed = -1 ;
  int i = -1 ;
  
  for (i = 0; i < nb_points; i++){
    speed  = sqrt (SQR(speeds[i].x) + SQR(speeds[i].y)) ;
    if (speed > THRESHOLD)
      return 0 ;
  }
  return 1 ;
}

void
compute_mirror_point (point * mirror_point, int i, point * points_array, 
		      int k, point * offset, int X, int Y) 
{
  switch ((int) offset[k].x){
    case -1:{
    mirror_point->x = - points_array[i].x ;
    break ;
  }
  case 0:{
    mirror_point->x = points_array[i].x ;
    break ;
  }
  case 1:{
    mirror_point->x = 2 * X - points_array[i].x ;
    break ;
  }
  }

  switch ((int) offset[k].y){
  case -1:{
    mirror_point->y = - points_array[i].y ;
    break ;
  }
  case 0:{
    mirror_point->y = points_array[i].y ;
    break ;
  }
  case 1:{
    mirror_point->y = 2 * Y - points_array[i].y ;
    break ;
  }
  }
}












