/* This software comprises contributed code made by Motorola, as a
 * Contributor, to Network Simulator NS-2 software provided by the
 * Regents of the University of California.
 * (Copyright; Regents of the University of California, 1994)
 * The contributed code was made as a result of a partnership between
 * Motorola and INRIA Rhone-Alpes.
 *
 * Copyright in the contributed code belongs to Motorola Inc. 2001
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ALL ADVERTISING MATERIALS MENTIONING FEATURES OR USE OF THIS SOFTWARE MUST
 * DISPLAY AN ACKNOWLEDGEMENT TO THE COPYRIGHT OWNERS.
 * ANY REDISTRIBUTION OF THIS SOFTWARE MUST CONTAIN THE ABOVE COPYRIGHT NOTICES,
 * CONDITIONS AND DISCLAIMER.
 */

/* ############################################################################
 * This code was developed by Mickael Vera (1999)
 * Modified by Thierry Ernst (2000)
 * MOTOROLA Labs Paris FRANCE and INRIA Rhone-Alpes FRANCE (Planete)
 * NS-2.1b6 enhancements for Wide-Area mobility simulations
 * ############################################################################
 */



#ifndef SGB2NS_SITES_H
#define SGB2NS_SITES_H

/* uncomment the following line to have an
   interactive interface to choose the anchors distribution */ 
/* #define INTERACTIVE */

/* user definitions */
#define TRANSIT_TRANSIT_BW "100Mb"
#define TRANSIT_BORDER_BW "50Mb"
#define INTRA_STUB_BW "10Mb"

#define TRANSIT_TRANSIT_DELAY_COEF 0.1 
#define TRANSIT_BORDER_DELAY_COEF  0.1 
#define INTRA_STUB_DELAY_COEF      0.1 

#define LINK_TAIL "DropTail"

/* end of user definitions */

#define MAX_NODES_NUMBER 10000

#define NS_INSTANCE "ns"
#define TM_INSTANCE "TOPOM"
#define TCL_INDENTATION "  "

#define TM_ADDR_INIT_PROC "tm_init_addr_params"
#define TM_BUILD_TOPO_PROC "tm_build_topo"
#define TM_BUILD_NODES_PROC "tm_load_nodes"
#define TM_BUILD_ANCHORS_PROC "tm_build_anchors"
#define TM_ADD_DOMAINS_AND_STUBS_PROC "tm_add_domains_and_stubs"
#define TM_BUILD_LINKS_PROC "tm_build_links"

#define TM_SET_ADDR_METHOD "tm_set_addr_params"
#define TM_BUILD_LINK_METHOD "tm_add_link"
#define TM_ADD_STUB_METHOD "tm_add_stub_to_tn"

#define TM_ADD_DOMAIN_METHOD "tm_add_nb_domains"
#define ADD_NB_STUB_METHOD "tm_add_nb_sites_to_tn_id"
#define ADD_TRANSIT_NODE_METHOD "tm_add_tn_to_domain"
#define ADD_STUB_NODE_METHOD "tm_add_node_to_site"
#define ADD_BORDER_NODE_METHOD "tm_add_border_to_site"
#define ADD_ANCHOR_NODE_METHOD "tm_add_anchor_router_node"
#define TM_BUILD_MOBILES_METHOD " tm_create_mobile_nodes"

#define TRUE 1
#define FALSE 0 
#define STUB_NAME_MAX_SIZE 100

/* address is the string d.c.n,
   it returns d.c */
char *
get_ip_prefix (char * address) ;

void 
usage (char * name) ;

/* translate addresses in the datas of the sgb graph
   into ns addresses and write partial information 
   in info_nodes

   addresses[n] : ns addresse of the node n
   domain : number of domains
   q[p] : nb of clusters in domain p
   node_per_cluster : string of nb of nodes for each cluster
   node_info[n] : info about node type TRANSIT_NODE or STUB_NODE
   stubs[i] : stub number i (2.3 for example)
   nb_stubs : total number fo stubs, size of the array stubs
*/

topology_map
create_ns_addresses (Graph * g) ;

/* write the function in the tcl file */
void
write_tm_init_addr (FILE * fout, topology_map T) ;

/* write the function in the tcl file */
void
write_tm_build_topo (FILE * fout, int X, int Y) ;

/* write the function in the tcl file */
void 
write_tm_add_domains_and_stubs (FILE * fout, Graph * g, topology_map Topology_map) ;


/* write the function in the tcl file and
   add information in info_nodes
   nodes are distinguished between STUB_NODES and BORDER_NODES

   transit_nodes : addresses of transit nodes
*/
void
write_tm_build_edges(FILE * fout, Graph * g, topology_map T) ;

/* write the function in the tcl file */
void
write_tm_build_nodes (FILE * fout, Graph * g, topology_map T) ;

/* write the function in the tcl file */
void
add_anchors (topology_map T, int anchors_number, int enable_random_anchor, 
		      int X, int Y, char * file_name) ;

/* returns the index of the stub of border_node_index in stubs */
int
get_stub_index (int border_node_index, char ** stubs, int nb_stubs, 
		char ** address) ;

/* return 1 if all chars of the string are digits, 0 if not */
int
is_number(char * str) ;

/* write the description fo the parameters that produced this script 
   for the human user */
void 
write_human_topology_info (FILE * fout, char * prog, int enable_random_anchor, 
			   int anchors_number, 
			   int X, int Y, char * sgb_file_name, char * tcl_file_name) ;

/* add a new address in the addresses domain in the stub stub,
   the domain is updated,
   the address is returned */
char *
get_new_address (int domain, int * clusters, char * node_per_cluster, 
		 char ** stubs, int stub) ;


#endif SGB2NS_SITES_H






