/* This software comprises contributed code made by Motorola, as a
 * Contributor, to Network Simulator NS-2 software provided by the
 * Regents of the University of California.
 * (Copyright; Regents of the University of California, 1994)
 * The contributed code was made as a result of a partnership between
 * Motorola and INRIA Rhone-Alpes.
 *
 * Copyright in the contributed code belongs to Motorola Inc. 2001
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ALL ADVERTISING MATERIALS MENTIONING FEATURES OR USE OF THIS SOFTWARE MUST
 * DISPLAY AN ACKNOWLEDGEMENT TO THE COPYRIGHT OWNERS.
 * ANY REDISTRIBUTION OF THIS SOFTWARE MUST CONTAIN THE ABOVE COPYRIGHT NOTICES,
 * CONDITIONS AND DISCLAIMER.
 */

/* ############################################################################
 * This code was developed by Mickael Vera (1999)
 * MOTOROLA Labs Paris FRANCE
 * NS-2.1b6 enhancements for Wide-Area mobility simulations
 * ############################################################################
 */


#include <malloc.h>
#include <assert.h>
#include <string.h>

#include "addresses.h"
#include "nodes_type.h"

#include "topology_map.h"


/* **************************************************************** 
   all_node structure 
   **************************************************************** */ 

struct all_nodes ;
typedef struct all_nodes * all_nodes ;

all_nodes
new_all_nodes () ;

int
add_node (all_nodes All_nodes, int Domain, int DomNode, int Stub_num, int Node_type, const char * option) ;

// XXX: have a array of struct instead
struct all_nodes {
  int Nb_nodes ;
  int Domain[MAX_TOPOLOGY_NODES];
  int DomNode[MAX_TOPOLOGY_NODES];
  int Stub[MAX_TOPOLOGY_NODES];
  address Addresses[MAX_TOPOLOGY_NODES] ;
  int Type[MAX_TOPOLOGY_NODES] ;
  char *options[MAX_TOPOLOGY_NODES] ;
} ;

all_nodes
new_all_nodes () {
  all_nodes All_nodes = (all_nodes) malloc (sizeof (struct all_nodes)) ;
  
  All_nodes->Nb_nodes = 0 ;
  return All_nodes ;
}

void
delete_all_nodes (all_nodes A) {
  int i ;
  for (i = 0; i < A->Nb_nodes; i++)
    if (A->options[i] != NULL)
      free (A->options[i]) ;
  free (A) ;
}

int
add_node (all_nodes All_nodes, int Domain, int DomNode, int Stub_num, int Node_type, const char * option) {
  assert (All_nodes->Nb_nodes < (MAX_TOPOLOGY_NODES - 1)) ;

  //All_nodes->Addresses[All_nodes->Nb_nodes] = Address ;
  All_nodes->Type[All_nodes->Nb_nodes] = Node_type ;
  All_nodes->Domain[All_nodes->Nb_nodes] = Domain ;
  All_nodes->DomNode[All_nodes->Nb_nodes] = DomNode ;
  All_nodes->Stub[All_nodes->Nb_nodes] = Stub_num ;
  if (option == NULL)
    All_nodes->options[All_nodes->Nb_nodes] = NULL ;
  else
    All_nodes->options[All_nodes->Nb_nodes] = strdup (option) ;
    
  All_nodes->Nb_nodes ++ ;
  
  return All_nodes->Nb_nodes ;
}

/* ****************************************************************
   topology_map structure 
   **************************************************************** */
struct topology_map {
  int Transit_domains_nb_nodes[MAX_DOMAINS] ;
  int Transit_nodes_nb_stubs[MAX_DOMAINS][MAX_TN_PER_DOMAIN] ;
  int Transit_nodes_stubs[MAX_DOMAINS][MAX_TN_PER_DOMAIN][MAX_NODES_PER_STUB] ;
   
  int topo_nb_stubs;
  all_nodes All_nodes ;
  addr_domain Addr_domain ;
} ;

topology_map
new_topology_map () {
/*  int i;  */
  topology_map T = (topology_map) malloc (sizeof (struct topology_map)) ;

  T->topo_nb_stubs = 0 ; 
  T->All_nodes = new_all_nodes () ;
  
  T->Addr_domain = new_addr_domain () ;

  /* create the domain for the transits nodes */
/*  i = new_domain (T->Addr_domain) ; assert (i == 0) ; */  

  return T ;
}

void
delete_topology_map (topology_map T) {
  delete_all_nodes (T->All_nodes) ;
  delete_addr_domain (T->Addr_domain) ;
  free (T) ;
}
 
int 
add_domain (topology_map T) {
  int new_free_domain ;
  new_free_domain = new_domain (T->Addr_domain) ;
  T->Transit_domains_nb_nodes[new_free_domain] = 0 ;

  return new_free_domain ;
}

int
add_transit_domain (topology_map T, int domain) {
  int new_transit_cluster ;

  new_transit_cluster = new_cluster(T->Addr_domain, domain) ;

  return new_transit_cluster ;
}

int
add_transit_node (topology_map T, int domain, int cluster) {
  int node ;
  
  assert (domain < MAX_DOMAINS) ;
  assert (get_nb_transit_nodes(T, domain) < 
	  (MAX_TRANSIT_NODE_PER_TRANSIT_DOMAIN - 1)) ;

  node = new_node(T->Addr_domain, domain, cluster) ;
  T->Transit_domains_nb_nodes[domain] ++ ;
  T->Transit_nodes_nb_stubs[domain][node] = 0 ;
  add_node(T->All_nodes, domain, cluster, NONE, TRANSIT_NODE, "") ;

  return node ;
}

int
add_stub (topology_map T, int domain, int node) {
  int nb_stubs ;
  assert (domain < MAX_DOMAINS) ;
  assert (node < MAX_TRANSIT_NODE_PER_TRANSIT_DOMAIN) ;
  nb_stubs = T->Transit_nodes_nb_stubs[domain][node] ;
  assert (nb_stubs < (MAX_STUB_PER_TRANSIT_NODE - 1)) ;
  
  T->Transit_nodes_nb_stubs[domain][node] ++;
  return ++(T->topo_nb_stubs) ;
}

void
add_anchor_node (topology_map T, int domain, char * option) {
  int new_free_cluster ;
  int node ;

  new_free_cluster = new_cluster (T->Addr_domain, domain) ;
  node = new_node (T->Addr_domain, domain, new_free_cluster) ;
  assert (node == 0) ;
  add_node (T->All_nodes, ERROR, ERROR, ERROR, ANCHOR_NODE, option) ;
}

int
add_stub_node (topology_map T, int domain, int dom_node, int stub) {
  int node ;

  assert (domain < MAX_DOMAINS) ;
  
  node = new_node (T->Addr_domain, domain, stub) ;
  add_node (T->All_nodes, domain, dom_node, stub, STUB_NODE, NULL) ;
  
  return (node) ;
}

int
add_stub_node_with_option (topology_map T, int domain, int type, const char * options) {
  int node ;
  
  assert (domain < MAX_DOMAINS) ;
  
  node = new_node (T->Addr_domain, domain, 0) ;
  add_node (T->All_nodes, ERROR, ERROR, ERROR, type, options) ;
  
  return (node) ;
}

int
topology_nb_nodes (topology_map T) {
  return (T->All_nodes->Nb_nodes) ;
}

int
node_type (topology_map T, int node_index) {
  assert (node_index < MAX_TOPOLOGY_NODES) ;

  return (T->All_nodes->Type[node_index]) ;
}
 
int
node_stub (topology_map T, int node_index) {
  assert (node_index < MAX_TOPOLOGY_NODES) ;

  return (T->All_nodes->Stub[node_index]) ;
}

int
node_dom_node (topology_map T, int node_index) {
  assert (node_index < MAX_TOPOLOGY_NODES) ;

  return (T->All_nodes->DomNode[node_index]) ;
}

int
node_domain (topology_map T, int node_index) {
  assert (node_index < MAX_TOPOLOGY_NODES) ;

  return (T->All_nodes->Domain[node_index]) ;
}

void
set_node_type (topology_map T, int node_index, int type) {
  assert (node_index < MAX_TOPOLOGY_NODES) ;
  
  T->All_nodes->Type[node_index] = type ;
}

char *
node_address (topology_map T, int node_index) {
  assert (node_index < MAX_TOPOLOGY_NODES) ;  

  return (address2string (T->All_nodes->Addresses[node_index])) ;
}

char *
node_option (topology_map T, int node_index) {
  assert (node_index < MAX_TOPOLOGY_NODES) ;
  
  return (T->All_nodes->options[node_index]) ;
} 

int
get_nb_transit_domains (topology_map T) {
  return (nb_domains(T->Addr_domain)) ;
}

int
get_nb_transit_nodes (topology_map T, int Transit_domain) {
  assert (Transit_domain < MAX_DOMAINS) ;

  return (T->Transit_domains_nb_nodes[Transit_domain]);
  // return (nb_nodes (T->Addr_domain, Transit_domain, DEFAULT_TN_CLUSTER)) ;
}

int
get_nb_stubs (topology_map T, int Transit_domain, int Node) {
  assert (Transit_domain < MAX_DOMAINS) ;
  assert (Transit_domain < get_nb_transit_domains(T)) ;
  assert (Node < MAX_TRANSIT_NODE_PER_TRANSIT_DOMAIN) ; 
  assert (Node < get_nb_transit_nodes(T, Transit_domain)) ;

  return (T->Transit_nodes_nb_stubs[Transit_domain][Node]) ;
}

/*
int
get_stub_address_domain (topology_map T, int Transit_domain, 
			 int Node, int stub_index) {
  assert (Transit_domain < MAX_DOMAINS) ;
  assert (Node < MAX_TRANSIT_NODE_PER_TRANSIT_DOMAIN) ;
  assert (stub_index < MAX_STUB_PER_TRANSIT_NODE) ;

  return (T->Transit_nodes_stubs[Transit_domain][Node][stub_index]) ;
}
*/

char *
get_topology_addresses_domain (topology_map T) {
  return (get_addr_domain (T->Addr_domain)) ;
}

