#!/bin/tcsh

# ######################################################################
# Demonstration of Mobile IPv6 for global mobility only
# Thierry Ernst
# Version 08.05.01
# ######################################################################
   set MOBI=.. 
   set topo=micro-test-20-0		# Topology to load
   set cn_nn=1				# nb of CNs
   set NS=ns-mipv6			# NS binary
   set bs_nn=1				# nb of Base Stations per site
   set nb_site=8			# nb of Sites where we want Base Stations
   set mt=500				# Movement time interval
   set stop=`expr $mt \* $nb_site`	# Stop time

   # NS Congiguration procedures call: 
   set mn_config="set-mobile SITE 1 150.00 150.00 0 0 0.01"
   set bs_config_file={$bs_nn}bs-{$nb_site}site.config
   set cn_config="select-cn ADDR 0 0 1.0.0 $stop"
   set cn_traffic="set-ping-int 1"
   set scen_config="set-scen NN_RAND $nb_site 0 $mt 0"

   # Input Directories
   set CONFD={$MOBI}/sim-config		# Directory for scenarios
   set TOPOD={$MOBI}/topologies 	# Directory for topologies
 
   # Input Files
   set CONFF={$CONFD}/{$bs_config_file}
   set TOPOF={$TOPOD}/{$topo}.tcl	# topology file path
  
   # Output Directory
   set OUTDIR=.tmp
   echo "Output in temporary directory $OUTDIR"
   if ( -d $OUTDIR ) then
	rm -R $OUTDIR
   endif
   mkdir $OUTDIR
   set NAMF={$OUTDIR}/out.nam		# NAM file (optional)
   set TRACEF={$OUTDIR}/out.tr
   set INFOF={$OUTDIR}/out.info   

   # Launch simulation  
   $NS mipv6.tcl -global ON -x 800 -y 800 -mactrace ON -topoman ALL -DISPLAY_INFO TRUE -DISPLAY_INPUT TRUE -cn_nn $cn_nn -cn_config "$cn_config" -cn_traffic "$cn_traffic" -bs_nn $bs_nn -bs_config_file $CONFF -mn_config "$mn_config" -scen_config "$scen_config" -stop $stop -topo_file $TOPOF -tracefile $TRACEF > $INFOF 

