#!/bin/tcsh

# ######################################################################
# Demonstration of Mobile IPv6 and local mobility 
# Thierry Ernst
# Version 08.05.01
# ######################################################################
   set MOBI=.. 
   set topo=small-topo-by-hand		# Topology to load
   set cn_nn=1				# nb of CNs
   set NS=ns-mipv6			# NS binary
   set bs_nn=4				# nb of Base Stations per site
   set nb_site=1			# nb of Sites where we want Base Stations
   set mt=500				# Movement time interval
   set stop=1000			# Stop time

   # NS Congiguration procedures call: 
   set mn_config="set-mobile SITE 1 10.00 10.00 0 1 0.01"
   set bs_config_file={$bs_nn}bs-{$nb_site}site.config
   set cn_config="select-cn ADDR 0 500 1.0.0 1000"
   set cn_traffic="set-ping-int 1"

   # Input Directories
   set CONFD={$MOBI}/sim-config		# Directory for scenarios
   set TOPOD={$MOBI}/topologies 	# Directory for topologies
 
   # Input Files
   set CONFF={$CONFD}/{$bs_config_file}
   set TOPOF={$TOPOD}/{$topo}.tcl	# topology file path
  
   # Output Directory
   set OUTDIR=.tmp
   echo "Output in temporary directory $OUTDIR"
   if ( -d $OUTDIR ) then
	rm -R $OUTDIR
   endif
   mkdir $OUTDIR
   set NAMF={$OUTDIR}/out.nam		# NAM file (optional)
   set TRACEF={$OUTDIR}/out.tr
   set INFOF={$OUTDIR}/out.info   

   # Launch simulation  
   $NS mipv6.tcl -x 800 -y 800 -mactrace OFF -topoman ALL -INFO TRUE -DISPLAY_INFO TRUE -cn_nn $cn_nn -cn_config "$cn_config" -cn_traffic "$cn_traffic" -bs_nn $bs_nn -bs_config_file $CONFF -mn_config "$mn_config" -stop $stop -topo_file $TOPOF -tracefile $TRACEF > $INFOF 
