/* This software comprises contributed code made by Motorola, as a 
 * Contributor, to Network Simulator NS-2 software provided by the 
 * Regents of the University of California.
 * (Copyright; Regents of the University of California, 1994)
 * The contributed code was made as a result of a partnership between 
 * Motorola and INRIA Rhone-Alpes. 
 *
 * Copyright in the contributed code belongs to Motorola Inc. 2001
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ALL ADVERTISING MATERIALS MENTIONING FEATURES OR USE OF THIS SOFTWARE MUST 
 * DISPLAY AN ACKNOWLEDGEMENT TO THE COPYRIGHT OWNERS. 
 * ANY REDISTRIBUTION OF THIS SOFTWARE MUST CONTAIN THE ABOVE COPYRIGHT NOTICES, 
 * CONDITIONS AND DISCLAIMER.
 */

/* ############################################################################
 * This code was developed by Thierry Ernst (1998-2001)
 * MOTOROLA Labs Paris FRANCE - INRIA Rhone-Alpes Grenoble (PLANETE) FRANCE 
 * NS-2.1b6 enhancements for Wide-Area mobility simulations
 * ############################################################################
 */

// *************************************************************************
//                              mipv6.h  
//      Mobility Support in IPv6 
//      Based on <draft-ietf-mobileip-ipv6-11.txt> March 2000 with some
//      simplifications.
//
// **************************************************************************
#ifndef ns_mipv6_h
#define ns_mipv6_h

#include "list.h"
#include "mip.h"

// **************************************************************************
// Mobile IPv6 Default Settings
// **************************************************************************
#define	MIP_TIMER_BU		0
#define	MIP_TIMER_BSLIST	1	
#define	NONE			-1

#define	INITIAL_BINDACK_TIMEOUT	1	// sec
#define	MAX_BINDACK_TIMEOUT	256	// sec
#define	MAX_UPDATE_RATE		1	// sec
#define	SLOW_UPDATE_RATE	10	// sec
#define	MAX_FAST_UPDATES	5	// transmissions
#define	MAX_ADVERT_REXMIT	3	// transmissions

#define BU_OPT_SIZE		10		// + sub_option
#define BU_ACK_OPT_SIZE		13		// + sub_option 
#define	BU_REQ_OPT_SIZE		2		// + sub_option
#define HOME_OPT_SIZE		18		// + sub_option  
#define	BU_SIZE	   DEST_HDR_PREFIX_SIZE + BU_OPT_SIZE + HOME_OPT_SIZE
#define BACK_SIZE  DEST_HDR_PREFIX_SIZE + BU_ACK_OPT_SIZE
#define BREQ_SIZE  DEST_HDR_PREFIX_SIZE + BU_REQ_OPT_SIZE	

typedef enum {
	BREQ, BU, BACK, 
	BU_HA, BU_BS,  BU_CN, BU_RP, BU_MN,
	BS_ADS, BS_SOL
} Mipv6RegType;

typedef enum {
        DEF_BU, LBM_BU, MBU_CN, MBU_ALL, LBM_BU_ALL, LBM_BU_CN, TEST
} IPv6OptionType;

#define ON			1	
#define OFF			0

// #define LIFETIME		min(reglftm_, adlftm_)
#define LIFETIME		reglftm_

#define TIME_INFINITY		0x0fffffff			

// **************************************************************************
// Binding Update List
// **************************************************************************
class Entry;
LIST_HEAD(entry, Entry);

class Entry {
 public:
        Entry(int address, Mipv6RegType t, int f)  { 
		addr = address;	
		haddr = -1;
		caddr = -1;
		seqno = -1; 
		flag  = f;
		lftm = 0;  
		time = -1;		
		active_expire = 0;
		nbbu = 0;
		type = t;
		
		switch ( t ) {
		   case BU_HA:
			strcpy(info, "HA");   break;
		   case BU_BS:
			strcpy(info, "BS");   break;
		   case BU_MN:
			strcpy(info, "MN");   break;
		   case BS_ADS:
			strcpy(info, "BS");   break;
		   case BU_RP:
			strcpy(info, "RP");   break;
		   default:
			strcpy(info, "CN");   break;
		}	
	}  
        ~Entry() { ; }

	// Chain element to the list
	inline void insert_entry(struct entry *head) {
        	LIST_INSERT_HEAD(head, this, link);
	}

	inline void update_entry(int seq, int h, int c, double t, double life) {
		seqno=seq;
		haddr=h;
		if ( c != NONE ) caddr=c;
		time=t;
		lftm=life;
		nbbu++;
		flag=ON;
	}
	inline double expire() { return (time + lftm) ; }
	inline double& lifetime() { return lftm; }
	inline int activated(double now) { 
		return ( (flag==ON) && (now<active_expire)) ? TRUE : FALSE; 
	}
	
	inline void activate_entry(double now, double life) {
	// We want to send BU and we want to keep doing this
	// until current time > active_expire_time 
		flag=ON;
		if ( (now+life) > active_expire) active_expire = now + life ;
	}

	// Return next element in the chained list
        Entry* next_entry(void) const { return link.le_next; }
	
	inline void remove_entry() { LIST_REMOVE(this, link); }

	inline void remove_entry(struct entry *newhead) {
	    LIST_REMOVE(this, link);
            LIST_INSERT_HEAD(newhead, this, link);
	}

	inline void deactivate_entry() {
		// We don't want to send BU to this node anymore
		// but we want to keep it in the list
		flag=OFF;
	}
	// Fields as described in the Mobile IPv6 spec.
       	int 		addr;		// address (key)
	int 		haddr;		// Last Home address sent
        int       	caddr;		// Last COA sent 
	double		lftm; 		// lifetime of this entry 
	double		time; 		// time at which entry was updated 
        int       	seqno;		// Last BU's sequence number
        int       	flag;		// Do not send BU if set to ON 

	// XXX: Other fields that may be included as in the spec: 
	// int       	state;		// State of retransmission

	// For statistic and display purposes only  
	double		active_expire;	// term of activation
	int 		nbbu;		// Total nb BU sent to this node
	Mipv6RegType	type;		// BU_HA / BU_CN / BU_REDIRECT ...  
	char		info[10];	// To distinguish between CN, HA, BS

 protected:
        LIST_ENTRY(Entry) link;
}; 

// **************************************************************************
// MIPv6 messages
// **************************************************************************
//b7: #define HDR_MIPv6(p)      (hdr_mipv6::access(p))
#define HDR_MIPv6(p)	((struct hdr_mipv6*)(p)->access(hdr_mipv6::offset_))
struct hdr_mipv6 {
	Mipv6RegType optype_;
	int A_;			// Ack required bit 
	int H_;			// Home registration bit
	int seq_; 
	double lftm_; 
	int homeaddr_;
	int coaddr_;	

	// XXX: Following will be needed for ns-2b.7 with some modification
	// - required by PacketHeaderManager
	static int offset_;
        inline static int& offset() { return offset_; }
        inline static hdr_mipv6* access(Packet* p) {
                return (hdr_mipv6*) p->access(offset_);
        }

	// New 02/01
 	inline Mipv6RegType& type() { return optype_; }
 	inline int& A() { return A_; }
 	inline int& H() { return H_; }
 	inline int& seqno() { return seq_; } 
 	inline double& lifetime() { return lftm_; }
 	inline int& coa() { return coaddr_; }
        inline int& haddr() { return homeaddr_; }
};

// **************************************************************************
// Timers
// **************************************************************************
class BUTimer : public TimerHandler {
 public:
        BUTimer(Agent *a) : TimerHandler() { a_ = a; }
 protected:
        inline void expire(Event *) { a_->timeout(MIP_TIMER_BU); }
        Agent *a_;
};

// Timer to reset BS's advertisement recorded at MN
class MNAgent;
class BSListTimer : public TimerHandler {
 public: 
        BSListTimer(MNAgent *a) : TimerHandler() { a_ = a; }
 protected:
        void expire(Event *e);
        MNAgent *a_;
};

// **************************************************************************
// Mobile IPv6 Base Agent
// **************************************************************************
class MIPv6Agent : public Agent {
public:
	MIPv6Agent();
	struct entry bcache_head_;	
protected:
	int command(int argc, const char*const*argv);
	NsObject *bcast_target_; 	// where to send solicitations / ads
	BUTimer timer_;
	double beacon_;			// Beacon interval
	int print_info_;
	char	node_info_[8];

	void send_packet(Packet *p);
	void set_ipv6_hdr(Packet *p, int);	
	void send_ack(Packet *p);
	Packet* set_ack_packet(int, int, int);
	void set_back_hdr(Packet *p, int);
	void encap_route(int, int, double);
	void decap_route(int, double);
	void delete_route(int);
	void print_info(Packet *p, char, char *);
	void recv(Packet *, Handler *);
	void mn_registration(Packet *p); 
	virtual void send_ads() { 
	        fprintf(stderr, "%s: Router Ads not allowed",node_info_); 
	}
	// Binding Cache
	Entry* update_bcache(Packet *, int);
 	// Entry* bcache_head()	{ return bcach_head_.lh_first; }

	// List Management
	Entry* lookup_entry(int, Entry *);	
 	Entry* head(struct entry head) { return head.lh_first; }

	// Display
	void dump();
	void dump_list(Entry *, char *);

};

// **************************************************************************
// Base Station Node (+HA)
// **************************************************************************
class BSAgent : public MIPv6Agent {
public:
	BSAgent();
	// virtual void recv(Packet *, Handler *);
	void timeout(int);
protected:
	int command(int argc, const char*const*argv);
	void send_ads();
	BUTimer timer_;
	double adlftm_;			// Router Advertisement lifetime 
	int rseq_;			// sequence number for routing ads
	NsObject *ragent_;     		// Routing or Network Agent 
};

// **************************************************************************
// Mobile IPv6 Node
// **************************************************************************
class MNAgent : public MIPv6Agent {
public:
	MNAgent();
	void recv(Packet *, Handler *);
	void timeout(int);
	struct entry bulist_head_;	
	struct entry bslist_head_;	
	struct entry history_head_;	

	void dump();

	// Binding Update List
	Entry* add_bulist(int, Mipv6RegType, int);
protected:
	int command(int argc, const char*const*argv);
	void recv_ads(Packet *);
	void recv_decap(Packet *);
	void recv_back(Packet *);
	void set_subnet();
	int get_coa(int);
	void reg();
	void send_sols();
	int ha_;
	int bs_;
	int oldbs_;  
	int coa_;
        int oldcoa_;
	int coalost_;
	int nbu_;
	double max_rate_;		// Max Binding Update interval
	double slow_rate_;		// Slow Binding Update interval
	int m_seqno_;		/* current registration seqno */
	double reglftm_;	/* registration lifetime */
	double adlftm_;		/* advertisement lifetime */
	MobileNode *node_;      /* ptr to my mobilenode,if appl. */
	int bu_policy_;	
	int rt_opti_;		// Route Optimization
	int bs_forwarding_;	// Previous BS forwarding
	int decap_port_;

	void send_standard_bu(Mipv6RegType type);
	Packet* set_bu_packet(Mipv6RegType, int, int);	
	void set_bu_hdr(Packet *p, Mipv6RegType, int);	

	// Base Station List
 	// Entry* bslist_head()	{ return bslist_head_.lh_first; }
	Entry* add_bs(int);
	BSListTimer bslist_timer_;

	// Binding Update List
 	// Entry* bulist_head()	{ return bulist_head_.lh_first; }
	BUTimer bu_timer_;
	void remove_bulist(int,  Mipv6RegType);

	// Display
	// void dump();
};

// **************************************************************************
// Mobile IPv6 Correspondent Node
// **************************************************************************
class CNAgent : public MIPv6Agent {
public:
        CNAgent();
      //  void recv(Packet *, Handler *);
        void timeout(int);
protected:
        int command(int argc, const char*const*argv);
        SimpleTimer timer_;

	// Display
	void dump();
};

#endif
