#
# This software comprises contributed code made by Motorola, as a 
# Contributor, to Network Simulator NS-2 software provided by the 
# Regents of the University of California.
# (Copyright; Regents of the University of California, 1994)
# The contributed code was made as a result of a partnership between 
# Motorola and INRIA Rhone-Alpes. 
#
# Copyright in the contributed code belongs to Motorola Inc. 2001
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
# ALL ADVERTISING MATERIALS MENTIONING FEATURES OR USE OF THIS SOFTWARE MUST 
# DISPLAY AN ACKNOWLEDGEMENT TO THE COPYRIGHT OWNERS. 
# ANY REDISTRIBUTION OF THIS SOFTWARE MUST CONTAIN THE ABOVE COPYRIGHT NOTICES, 
# CONDITIONS AND DISCLAIMER.
#
#
# ############################################################################
# This code was developed by Thierry Ernst (1998-2001)
# MOTOROLA Labs Paris FRANCE - INRIA Rhone-Alpes Grenoble (PLANETE) FRANCE 
# NS-2.1b6 enhancements for Wide-Area mobility simulations
# ############################################################################


# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# MOBIWAN modifications
# I need get-node-addr to return MN's addr.
# I can not add MN in array Node_, then MNs are grouped in array MNode_
# Get-node-addr searches both arrays.
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Simulator instproc node args {
        $self instvar Node_ routingAgent_ wiredRouting_ 
	$self instvar MNode_ 
        if { [Simulator info vars EnableMcast_] != "" } {
                warn "Flag variable Simulator::EnableMcast_ discontinued.\n\t\
                      Use multicast methods as:\n\t\t\
                        % set ns \[new Simulator -multicast on]\n\t\t\
                        % \$ns multicast"
                $self multicast
                Simulator unset EnableMcast_
        }
        if { [Simulator info vars NumberInterfaces_] != "" } {
                warn "Flag variable Simulator::NumberInterfaces_ discontinued.\n\t\
                      Setting (or not) this variable will not affect the simulations."
                Simulator unset NumberInterfaces_
        }

        # wireless-ready node
        if [info exists routingAgent_] {
            if  {[string compare $routingAgent_ ""] != 0} {
                set node [$self create-wireless-node $args]

                # for base node

                if {[info exists wiredRouting_] && $wiredRouting_ == "ON"} {

                   set Node_([$node id]) $node
                } else {
                   set MNode_([$node id]) $node
		}   
                return $node
            }
        }

        set node [new [Simulator set node_factory_] $args]

        set Node_([$node id]) $node

        $node set ns_ $self
# XXX: now call it in makemip directly              
# Multicast can not work without mip (unless we just add a test here ...)
#        if [$self multicast?] {
#                $node enable-mcast $self
#        }
        $self check-node-num
        return $node
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Enhanced to return Mobile Node's object given its id 
# Because MNs are not recorded in array Node_
# MNs are recorded in array MNode_
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Simulator instproc get-node-by-id id {
        $self instvar Node_ MNode_

	if [info exists Node_($id)] {
        	set Node_($id)
	} else {
		set MNode_($id)
	}
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# MOBIWAN modification
# Enhanced to return IPv6 node type BS and MN 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Simulator instproc get-nodetype {} {

      $self instvar addressType_ routingAgent_ wiredRouting_ 
      set val ""

    if { [info exists addressType_] && $addressType_ == "hierarchical" } {
        set val Hier
    }
   
    if { [info exists routingAgent_] && $routingAgent_ != "" } {
        set val Mobile
    }   

    if { [info exists wiredRouting_] && $wiredRouting_ == "ON" } {
        set val Base
    }   
 
    if { [info exists wiredRouting_] && $wiredRouting_ == "OFF"} {
        set val Base
    }   

    if { [Simulator set mobile_ip_] } {
        if { $val == "Base" && $wiredRouting_ == "ON" } {
            set val MIPBS
        }

        if { $val == "Base" && $wiredRouting_ == "OFF" } {
            set val MIPMH
        }     
    }  

   if [$self mipv6?] {
	# set val MIPv6
	if { [info exists wiredRouting_] && $wiredRouting_ == "ON" } {
	   set val BS
	}
	if { [info exists wiredRouting_] && $wiredRouting_ == "OFF" } {
	   set val MN
	}
   }
    return $val  
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# MOBIWAN modification
# Enhanced to give nb of bits for each level as parameters 
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Simulator instproc addressType { type args } {
	# Useless when we set it  
	# $self instvar def_hier_addr_
	$self set addressType_ $type 
	if { [llength $args] == 0 } {
		$self set def_hier_addr_ "" 
	} else {
		$self set def_hier_addr_ $args 
	}
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# MOBIWAN modifications
# Enhanced to support Mobile Nodes and 4 levels addressing
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Simulator instproc node-config args {
        set args [eval $self init-vars $args]
        $self instvar  addressType_  routingAgent_ nodefactory_ propType_
        $self instvar macTrace_ routerTrace_ agentTrace_ movementTrace_
        $self instvar channelType_ topoInstance_ propInstance_ chan mobileIP_
        $self instvar rxPower_ txPower_

        if [info exists macTrace_] {
            Simulator set MacTrace_ $macTrace_
        }


        if [info exists routerTrace_] {
            Simulator set RouterTrace_ $routerTrace_
        }


        if [info exists agentTrace_] {
            Simulator set AgentTrace_ $agentTrace_
        }

        if [info exists movementTrace_] {
            Simulator set MovementTrace_ $movementTrace_
        }

        # hacking for matching old cmu add-interface
        # not good style, for back-compability ONLY

        # Only create 1 instance of prop
        if {[info exists propType_] && ![info exists propInstance_]} {
            set propInstance_ [new $propType_]
        }

        if {[info exists channelType_] && ![info exists chan]} {
            set chan [new $channelType_]
        }

        if [info exists topoInstance_] {
            $propType_  topography $topoInstance_
        }
# set address type, hierarchical or expanded

    if {[string compare $addressType_ ""] != 0} {
	   $self instvar def_hier_addr_
	# XXX: extension to speficy our own addressing
	# We need to pass nb level + nb bits/level to set-address-format.
        eval $self set-address-format $addressType_ $def_hier_addr_ 
#          eval $self set-address-format $addressType_ 
        }

# set mobileIP flag
    if { [info exists mobileIP_] && $mobileIP_ == "ON"} {

           Simulator set mobile_ip_  1
    } else {
           if { [info exists mobileIP_] } {
               Simulator set mobile_ip_ 0
           }
    }

}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# MOBIWAN modification
# Enhanced in order to replace Ad-Hoc routing Agent with Network Agent
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Simulator instproc create-wireless-node { args } {
        $self instvar routingAgent_ wiredRouting_
        $self instvar propInstance_ llType_ macType_ ifqType_ ifqlen_ phyType_ chan
        $self instvar antType_ energyModel_ initialEnergy_ txPower_ rxPower_
        $self instvar imepflag_ topoInstance_
        $self instvar namtraceAllFile_
        $self instvar level1_ level2_


        set imepflag_ OFF

        # create node instance

        set node [$self create-node-instance $args]

        if { [info exist namtraceAllFile_] } {
            $node get-nam-traceall $namtraceAllFile_
        }
   
        # basestation address setting
        if { [info exist wiredRouting_] && $wiredRouting_ == "ON" } {
            $node base-station [AddrParams set-hieraddr [$node node-addr]]
        }

        switch -exact $routingAgent_ {
	    Network {
                set ragent [$self create-network-agent $node]
	    }
            DSDV {
                set ragent [$self create-dsdv-agent $node]
            }
                
            DSR {
                $self at 0.0 "$node start-dsr"
            }

            AODV {
                set ragent [$self create-aodv-agent $node]
            }

            TORA {
                set imepflag_ ON
                set ragent [$self create-tora-agent $node]

            }
            default {
                puts "Wrong node routing agent!"
                exit
            }

        }
 # add main node interface
	$self instvar iface_
	# We don't always need the full network stack (MAC / ARP / LL etc)
        if { [info exist iface_] && $iface_ == "LIGHT" } {
	   # XXX: For Testing - We don't use the full network stack
	   # XXX: Basically, we want to avoid a bug due to MAC when the 
	   # XXX mobile node switches channel. 
           $node add-interface-light $chan $propInstance_ $llType_ $macType_ \
               $ifqType_ $ifqlen_ $phyType_ $antType_
        } elseif { [info exist iface_] && [[Simulator instance] get-nodetype] == "MN" && $iface_ == "DUAL" } {
	   # MN with 2 interfaces.
	   # For inter-site mobility
           $node add-interface $chan $propInstance_ $llType_ $macType_ \
               $ifqType_ $ifqlen_ $phyType_ $antType_
	   set nulchan [set_null_channel]
           $node add-interface $nulchan $propInstance_ $llType_ $macType_ \
               $ifqType_ $ifqlen_ $phyType_ $antType_
	} else {
	   # We use the full network stack
           $node add-interface $chan $propInstance_ $llType_ $macType_ \
               $ifqType_ $ifqlen_ $phyType_ $antType_
	}
        # attach agent

        if {$routingAgent_ != "DSR"} {
             $node attach $ragent 255
        }

        # bind routing agent and mip agent if existing
        # basestation address setting

        if { [info exist wiredRouting_] && $wiredRouting_ == "ON" } {
            if { $routingAgent_ != "DSR" } {
                $node mip-call $ragent
            }
        }

        #
        # This Trace Target is used to log changes in direction
        # and velocity for the mobile node.
        #

        set tracefd [$self get-ns-traceall]

        if {$tracefd != "" } {

            $node nodetrace $tracefd
            $node agenttrace $tracefd

        }
        set namtracefd [$self get-nam-traceall]

        if {$namtracefd != "" } {

            $node namattach $namtracefd
        }

        if [info exists energyModel_] {
             if  [info exists level1_] {
               set l1 $level1_
             } else {
               set l1 0.5
             }
             if  [info exists level2_] {
               set l2 $level2_
             } else {
               set l2 0.2
             }
             $node addenergymodel [new $energyModel_ $initialEnergy_ $l1 $l2]
        }

        if [info exists txPower_] {
            $node setPt $txPower_
        }

        if [info exists rxPower_] {
            $node setPr $rxPower_
        }

        $node topography $topoInstance_
        return $node

}

