# ############################################################################
# This code was developed by Thierry Ernst (1998-2001)
# MOTOROLA Labs Paris FRANCE - INRIA Rhone-Alpes Grenoble (PLANETE) FRANCE 
# NS-2.1b6 enhancements for Wide-Area mobility simulations
# ############################################################################

# ############################################################################
# Modifications to allow Mobile Nodes to support Multicast 
# Tested with CtrMcast only
# ############################################################################

# ############################################################################
# Existing proc that I need to modify from file /tcl/ctr-mcast/CtrMcast.tcl:
# ############################################################################
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# CtrMcastComp instproc set_c_rp args 
# To allow a mobile node to join a CBT mcast protocol
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
CtrMcastComp instproc set_c_rp args {
        $self instvar ns_

        foreach n [$ns_ ww-nodes] {
                set arbiter [$n getArbiter]
                #NS creates a virtual node per LAN.  This node does
                #not have any the multicast features set.
                if {$arbiter != ""} {
                        set ctrmcast [$arbiter getType "CtrMcast"]
                        $ctrmcast instvar c_rp_
                        $ctrmcast unset_c_rp
                }
        }

        foreach node $args {
                set arbiter [$node getArbiter]
                set ctrmcast [$arbiter getType "CtrMcast"]
                $ctrmcast set_c_rp
        }
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# CtrMcast instproc handle-cache-miss { srcID group iface } 
# To allow a mobile node to join a CBT mcast protocol
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
CtrMcast instproc handle-cache-miss { srcID group iface } {
        $self instvar ns_ agent_ node_
        $self instvar defaultTree_

        if { [$agent_ treetype? $group] == "" } {
                $agent_ treetype $group $defaultTree_
                #$agent_ add-new-member $group $node_
         }
        if { [$node_ id] == $srcID } {
                # XXX Begin Enhancement
                if [$ns_ is-mobile? [$node_ id]] {
                   #set bs [$ns_ get-node-by-id [$node_ set bs_]]
                   set bs [$ns_ get-node-by-addr [$node_ get-bs]]
                   set RP [[[$bs getArbiter] getType "CtrMcast"] get_rp $group]
                } else {
                   set RP [$self get_rp $group]
                }
                # set RP [$self get_rp $group]
                # XXX End Enhancement    
                if {[$agent_ treetype? $group] == "RPT" && $srcID != [$RP id]} {
                        set encapagent [new Agent/Encapsulator]
                        $ns_ attach-agent $node_ $encapagent

                        set ctrmcast [[$RP getArbiter] getType "CtrMcast"]
                        $ns_ connect $encapagent [$ctrmcast set decapagent_]

                        ### create (S,G,iif=-1) entry
                        $node_ add-mfc-reg $srcID $group -1 $encapagent
                }

                if [$agent_ new-source? $group $node_] {
                        $agent_ compute-tree $node_ $group
                }
        } elseif [SessionSim set MixMode_] {
            set srcnode [$ns_ get-node-by-id $srcID]
            if [$agent_ new-source? $group $srcnode] {
                $agent_ compute-tree $srcnode $group
            }
        }
        return 1 ;#call again
}

# ############################################################################
# New procs
# ############################################################################
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# About same procedure as Node instproc enable-mcast
# but also allow hierarchical addressing at the same time
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Node/MobileNode instproc enable-mcast sim {
        $self instvar classifiers_ multiclassifier_ ns_ switch_ mcastproto_
        $self set ns_ $sim

        $self set switch_ [new Classifier/Addr]
        #
        # set up switch to route unicast packet to slot 0 and
        # multicast packets to slot 1
        #

        [$self set switch_] set mask_ [AddrParams set McastMask_]
        [$self set switch_] set shift_ [AddrParams set McastShift_]

        #
        # create a classifier for multicast routing
        #
        $self set multiclassifier_ [new Classifier/Multicast/Replicator]
        [$self set multiclassifier_] set node_ $self

        $self set mrtObject_ [new mrtObject $self]

        $switch_ install 0 $classifiers_(1)
        $switch_ install 1 $multiclassifier_
}

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# This proc enable LBMcast or traditional multicast if relevant 
# flag is set
# XXX: cannot use both types simultaneously
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Node instproc test-mcast {} { 
   $self instvar classifiers_

    $self instvar ns_
    if ![info exist ns_] {
        set ns_ [Simulator instance]
    }
    if [$ns_ multicast?] {
                $self enable-mcast $ns_
    }
    if [$ns_ lbmcast?] {
                $self enable-lbmcast 
    }
}
