/* This software comprises contributed code made by Motorola, as a 
 * Contributor, to Network Simulator NS-2 software provided by the 
 * Regents of the University of California.
 * (Copyright; Regents of the University of California, 1994)
 * The contributed code was made as a result of a partnership between 
 * Motorola and INRIA Rhone-Alpes. 
 *
 * Copyright in the contributed code belongs to Motorola Inc. 2001
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ALL ADVERTISING MATERIALS MENTIONING FEATURES OR USE OF THIS SOFTWARE MUST 
 * DISPLAY AN ACKNOWLEDGEMENT TO THE COPYRIGHT OWNERS. 
 * ANY REDISTRIBUTION OF THIS SOFTWARE MUST CONTAIN THE ABOVE COPYRIGHT NOTICES, 
 * CONDITIONS AND DISCLAIMER.
 */

/* ############################################################################
 * This code was developed by Thierry Ernst (1998-2001)
 * MOTOROLA Labs Paris FRANCE - INRIA Rhone-Alpes Grenoble (PLANETE) FRANCE 
 * NS-2.1b6 enhancements for Wide-Area mobility simulations
 * ############################################################################
 */


// *************************************************************************
//                              ipv6.h
//      IPv6 Extensions  
//
// **************************************************************************

#ifndef ipv6_h
#define ipv6_h

#include "mip.h"
#include "ip.h"
#include "connector.h"
#include "mipv6.h"

// Do I need the 3 following ?
#include "config.h"
#include "agent.h"
#include "mac.h"


#define IPv6_ADDR_SIZE 		16
#define IPv6_HEADER_SIZE 	2 * IPv6_ADDR_SIZE + 8 
#define DEST_HDR_PREFIX_SIZE	2
#define IP_HEADER_SIZE IPv6_HEADER_SIZE
#define UDP_HDR_SIZE		8

// **************************************************************************
// IPv6 Router advertisements and Solicitations
// RFC2461
// XXX: Soliciations does not need a specific packet format
// **************************************************************************
#define HDR_RTADS(p)    ((struct hdr_rtads*)(p)->access(hdr_rtads::offset_))
struct hdr_rtads {
        double lftm_;
	// I should add the prefix as an option

        static int offset_;
        inline static int& offset() { return offset_; }
        inline static hdr_rtads* access(Packet* p) {
                return (hdr_rtads*) p->access(offset_);
        }
        inline double& lifetime() { return lftm_; }
};

#define PT_RSOL_SIZE		IPv6_HEADER_SIZE + 8
#define PT_RADS_SIZE		IPv6_HEADER_SIZE + 16

// **************************************************************************
// IPv6 Routing Header
// **************************************************************************
#define ROUTE_HDR_SIZE		16
#define IPv6_RTHDR_SIZE(n)	16 * n + 32 	

// Multiple hops OK
typedef struct ns_hop_list {
	ns_addr_t       dst_;	
	struct ns_hop_list *next_;
} ns_hop;
	
#define HDR_ROUTING(p)	((struct hdr_rtext*)(p)->access(hdr_rtext::offset_))
struct hdr_rtext {
	ns_hop *nexthop_;    // Next hop, not "next header"	
	int nb_;
	static int offset_;
        inline static int& offset() { return offset_; }

	inline static hdr_rtext* access(Packet* p) {
                return (hdr_rtext*) p->access(offset_);
        }
	inline int& nb() { return nb_ ; }
};


// >----------------------------------------------------------------------<
// >                          Network Agent                               <
// >----------------------------------------------------------------------<
// taken from dsdv agent

class Network : public Agent {
public:
  Network();
  virtual int command(int argc, const char * const * argv);

protected:
  void sendOutBCastPkt(Packet *p);
  int myaddr_;              // My address...
  MobileNode *node_;        // My node
  NsObject *port_dmux_;     // my port dmux
  int print_info_;
};

// >----------------------------------------------------------------------<
// >                       Network BS Agent                               <
// >----------------------------------------------------------------------<
class NetworkBS : public Network {
public:
  NetworkBS();
  virtual int command(int argc, const char * const * argv);

protected:
  virtual void recv(Packet *, Handler *);
};

// >----------------------------------------------------------------------<
// >                       Network MN Agent                               <
// >----------------------------------------------------------------------<
class NetworkMN : public Network {
public:
  NetworkMN();
  virtual int command(int argc, const char * const * argv);

protected:
  virtual void recv(Packet *, Handler *);
  MNAgent *mipagent_;
  int decap_port_;
  int off_ipinip_;
};

#endif

