/* This software comprises contributed code made by Motorola, as a 
 * Contributor, to Network Simulator NS-2 software provided by the 
 * Regents of the University of California.
 * (Copyright; Regents of the University of California, 1994)
 * The contributed code was made as a result of a partnership between 
 * Motorola and INRIA Rhone-Alpes. 
 *
 * Copyright in the contributed code belongs to Motorola Inc. 2001
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ALL ADVERTISING MATERIALS MENTIONING FEATURES OR USE OF THIS SOFTWARE MUST 
 * DISPLAY AN ACKNOWLEDGEMENT TO THE COPYRIGHT OWNERS. 
 * ANY REDISTRIBUTION OF THIS SOFTWARE MUST CONTAIN THE ABOVE COPYRIGHT NOTICES, 
 * CONDITIONS AND DISCLAIMER.
 */

/* ############################################################################
 * This code was developed by Thierry Ernst (1998-2001)
 * MOTOROLA Labs Paris FRANCE - INRIA Rhone-Alpes Grenoble (PLANETE) FRANCE 
 * NS-2.1b6 enhancements for Wide-Area mobility simulations
 * ############################################################################
 */

// *************************************************************************
// 				mipv6.cc  
//      Mobility Support in IPv6 
//	Based on <draft-ietf-mobileip-ipv6-11.txt> March 2000 with some
// 	simplifications.
//
// **************************************************************************
#define MYNUM	Address::instance().print_nodeaddr(addr())
#define MIPHAD	Address::instance().print_nodeaddr(miph->haddr())
#define PRINTADDR(a)     Address::instance().print_nodeaddr(a)
#define GETADDR(a)     Address::instance().get_nodeaddr(a)
#define HADDR	Address::instance().get_nodeaddr(addr())
#define MIPv6DEBUG

#include <template.h>
#include <random.h>
#include <address.h>
#include <mobilenode.h>
#include <assert.h>

#include "list.h"
#include "ipv6.h"
#include "mipv6.h"

// *************************************************************************
// >----------------------------------------------------------------------<
// Mobile IPv6 Binding Update Header
// >----------------------------------------------------------------------<
// *************************************************************************
int hdr_mipv6::offset_;
static class MIPv6HeaderClass : public PacketHeaderClass {
public:
	MIPv6HeaderClass() : PacketHeaderClass("PacketHeader/MIPv6",
					     sizeof(hdr_mipv6)) {
	    bind_offset(&hdr_mipv6::offset_);
        }
        void export_offsets() {
                field_offset("coa_", OFFSET(hdr_mipv6, coa()));
        }
} class_mipv6hdr;

// *************************************************************************
// >----------------------------------------------------------------------<
// MIPv6 Base Agent
// >----------------------------------------------------------------------<
// *************************************************************************
static class MIPv6AgentClass : public TclClass {
public:
        MIPv6AgentClass() : TclClass("Agent/MIPv6") {}
        TclObject* create(int, const char*const*) {
                return (new MIPv6Agent());
        }
} class_mipv6agent;


MIPv6Agent::MIPv6Agent() : Agent(PT_UDP), bcast_target_(0), timer_(this),
beacon_(1.0), print_info_(0)
{
	// Binding Cache - mostly to collect statistics
        LIST_INIT(&bcache_head_);
        
	bind("print_info_", &print_info_);
}

// *************************************************************************
// Recipient may altrenatively be a Base Station, a Home Agent, or a 
// simple Correspondent Node
// *************************************************************************
void MIPv6Agent::recv(Packet* p, Handler *)
{
   hdr_mipv6 *miph = HDR_MIPv6(p);
   hdr_cmn *hdrc = HDR_CMN(p);

   switch (hdrc->ptype()) {
      case PT_BU:
          if (miph->H() == ON) {
	     // Home Registration
	     // XXX: No distinction in the simulation
	     strcpy( node_info_, "HA");
	     mn_registration(p);
	  } else
	     mn_registration(p);
	  break;

      case PT_SOL:
	  // Process Solicitations from MN
	  send_ads();
	  timer_.resched(Random::uniform(0, beacon_));
	  //timer_.resched(beacon_);
	  break;

      case PT_RADS:
 	  // Advertisement from other BSs.  Just ignore
	  break;

      default:
	  fprintf(stderr, "%s received packet with wrong type\n", node_info_);
	  // exit(1); 
	  break;
	}
   Packet::free(p);
}

// *************************************************************************
// Send the Mobile IPv6 Packet  
// *************************************************************************
void MIPv6Agent::send_packet(Packet *p)
{
   hdr_cmn *hdrc = HDR_CMN(p);
   hdr_mipv6 *miph = HDR_MIPv6(p);
   hdr_ip *iph = HDR_IP(p);

   if (print_info_) {
      switch (hdrc->ptype()) {
	case PT_BACK:
	   cout << "+ " << node_info_ << " " << MYNUM << " send ACK [" 
	   << miph->seqno() << "] to " << PRINTADDR(iph->daddr()) << "\n";
	   break;

	case PT_BU:
	    cout << "+ " << node_info_ << " " << MYNUM << " send BU [" 
	    << miph->seqno() << "] to " << PRINTADDR(iph->daddr()) << "\n";
	   break;

	default:
	   cout << "+ " << node_info_ << " " << MYNUM 
	   << " send BU [" << miph->seqno() 
	   << "] to " << PRINTADDR(iph->daddr()) << "\n";
	   break;
	}
   }
   send(p, 0); 
}

// *************************************************************************
// XXX: temporary.
// Encapsulator should make use of Binding Cache.  This still require
// some modification.  Will do it later.
// *************************************************************************
void MIPv6Agent::encap_route(int fromaddr, int toaddr, double lft)
{
   Tcl& tcl = Tcl::instance();
   tcl.evalf("%s encap-route %d %d %lf", name_, fromaddr, toaddr, lft);
}

// *************************************************************************
// *************************************************************************
void MIPv6Agent::delete_route(int addr) 
{
   Tcl& tcl = Tcl::instance();
   tcl.evalf("%s clear-reg %d", name_, addr);
}

// *************************************************************************
// *************************************************************************
void MIPv6Agent::decap_route(int addr, double lft) 
{
// XXX: MN as it is built in TCL does not need a call to decap-route
// XXX: since packets are encapsulated to port DECAP_PORT (=1)
// XXX: Decapsulator handles packet to the default target i.e. node entry. 
// Tcl& tcl = Tcl::instance();
// tcl.evalf("%s decap-route %d %lf", name_, addr, lft);
}

// *************************************************************************
// CN or former default router (previous BS) is instructed 
// to redirect DTG towards current COA
// If CoA = Home Address, this is a de-registration (probably back to home)
// *************************************************************************
void MIPv6Agent::mn_registration(Packet *p)
{
   hdr_mipv6 *miph = HDR_MIPv6(p);

   if (miph->coa() == miph->haddr()) { 
     // De-registration 
     print_info(p, '-', "DEREG");
     update_bcache(p, OFF);
   }
   else {
     // Registration 
     print_info(p, '-', "REG");
     update_bcache(p, ON);
   }    
   if (miph->A() == ON)
	send_ack(p);
}

// *************************************************************************
// *************************************************************************
void MIPv6Agent::send_ack(Packet *p)
{
   hdr_mipv6 *miph = HDR_MIPv6(p);
   send_packet(set_ack_packet(miph->haddr(), miph->coa(), miph->seqno()));
}

// *************************************************************************
// Set the packet containing the Binding Acknowledgment
// *************************************************************************
Packet* MIPv6Agent::set_ack_packet(int final_dst, int via_hop, int seqno)
{
   Packet *p = allocpkt();
   set_ipv6_hdr(p, final_dst);   
   set_back_hdr(p, seqno);     
   return(p);
}

// *************************************************************************
// Fill the Binding Acknowledgment Option
// *************************************************************************
void MIPv6Agent::set_back_hdr(Packet *p, int seqno)
{
   hdr_mipv6 *h = HDR_MIPv6(p);
   hdr_cmn *hdrc = HDR_CMN(p);

   hdrc->size() += BACK_SIZE;
   hdrc->ptype() = PT_BACK;
   h->type() = BACK;
   h->seqno() = seqno;
   h->lifetime() = 0; 
}

// *************************************************************************
// Fill the IPv6 header
// *************************************************************************
void MIPv6Agent::set_ipv6_hdr(Packet *p, int dst)
{
   hdr_cmn *hdrc = HDR_CMN(p);

   // saddr() is set in Agent::initpkt
   // Rewrite IPv4 header size set in Packet::initpkt 
   hdrc->size() = IPv6_HEADER_SIZE;
   hdr_ip *iph = HDR_IP(p);
   iph->daddr() = dst;
   iph->dport() = port();
}

// *************************************************************************
// Fill the Routing header
// XXX: Now useless: done by SrcRouting which adds Routing Header
// *************************************************************************
/*
void MIPv6Agent::set_routing_hdr(Packet *p, int final_dst, int via_hop)
{
   hdr_mipv6 *h = HDR_MIPv6(p);
   hdr_cmn *hdrc = HDR_CMN(p);
   hdrc->size() += ROUTE_HDR_SIZE;
   h->haddr() = final_dst;
   h->coa() = via_hop;
}
*/

// *************************************************************************
// Print info to follow up how the protocole behave  
// c: '+' indicates send packet / '-' indicates received.
// *************************************************************************
void MIPv6Agent::print_info(Packet *p, char c, char *txt) 
{
   hdr_mipv6 *miph = HDR_MIPv6(p);

   if (print_info_) {
      cout << c << " " << node_info_ << " " << MYNUM << " " << txt 
      << " BU [" << miph->seqno() << "] from " << PRINTADDR(miph->haddr())
      << " (" << PRINTADDR(miph->coa()) << ")" << " at " << NOW << "\n";
   }
}

// *************************************************************************
int MIPv6Agent::command(int argc, const char*const* argv)
{
   if (argc == 2) {
      if (strcmp(argv[1], "dump") == 0) {
	dump();
	return TCL_OK;
      }
   }
   if (argc == 3) {
	if (strcmp(argv[1], "bcast-target") == 0) {
		bcast_target_ = (NsObject *)TclObject::lookup(argv[2]);
		return TCL_OK;
	}
   }
   return (Agent::command(argc, argv));
}

// *************************************************************************
// Print:
// Binding Cache
// *************************************************************************
void MIPv6Agent::dump()
{
   dump_list(head(bcache_head_), "Binding Cache");
}

// *************************************************************************
// Print the content of the list
// All lists have the same format
// *************************************************************************
void MIPv6Agent::dump_list(Entry *node, char *txt) {

   if ( node ) {
	cout << "\n|" << txt << " for node " << PRINTADDR(addr())
	<<  " at " << NOW << " --------------------------------------- |\n";
	cout << "|Node\t\tCOA\t\tType\tInfo\tFlag\tLast\tTime\t\tLife\t"
	<< "Expire\t\tNb|\n"; 

	for ( ; node; node = node->next_entry()) {
	
     	  cout << "|" <<  PRINTADDR(node->addr) << "\t\t" 
	  << PRINTADDR(node->caddr) << "\t\t" << node->type 
     	  << "\t" << node->info << "\t" << node->flag  << "\t"
     	  << node->seqno  << "\t" << node->time << "\t\t"
	  << node->lifetime() << "\t" 
	  << node->active_expire << "\t\t" << node->nbbu << " |\n";
        }
	cout << "\n";
   }
}

// *************************************************************************
// Search for an entry in the list pointed by its first element 
// *************************************************************************
Entry* MIPv6Agent::lookup_entry(int addr, Entry *n)
{
   for ( ; n ; n=n->next_entry()) {
       if(n->addr == addr)
             return n;
   }
   return NULL;
}

// *************************************************************************
// Add a new node in the Binding Cache
// XXX: Binding Cache should be used by Encapsulator and Decapsulator objects
// XXX  direclly - this will be added later.
// XXX  Now, Binding Cahce is only used to dump what is received by a node 
// *************************************************************************
Entry* MIPv6Agent::update_bcache(Packet *p, int flag)
{
   hdr_mipv6 *h = HDR_MIPv6(p);

   // Check if an entry does already exists for the source address
   Entry *n = lookup_entry(h->haddr(), head(bcache_head_));

   if ( !n ) {
      n = new Entry(h->haddr(), BU_MN, flag);
      n->insert_entry(&bcache_head_);
   }

   if ( flag == ON )
     encap_route(h->haddr(), h->coa(), h->lifetime());
   else 
     delete_route(h->haddr());
   n->update_entry(h->seqno(), h->haddr(), h->coa(), NOW, h->lifetime());
   
   return n;
}

// *************************************************************************
// >----------------------------------------------------------------------<
// MIPv6 Base Stations and Home Agent
// >----------------------------------------------------------------------<
// *************************************************************************
static class BSAgentClass : public TclClass {
public:
	BSAgentClass() : TclClass("Agent/BS") {}
	TclObject* create(int, const char*const*) {
		return (new BSAgent());
	}
} class_bsagent;

// *************************************************************************
BSAgent::BSAgent() : MIPv6Agent(), timer_(this), 
adlftm_(~0), rseq_(0), ragent_(0)
{
	bind("ad_lifetime_", &adlftm_);
	strcpy(node_info_, "BS");
}

// *************************************************************************
void BSAgent::timeout(int )
{
	send_ads();
	timer_.resched(Random::uniform(0, beacon_));
	//timer_.resched(beacon_);
}

// *************************************************************************
int BSAgent::command(int argc, const char*const* argv)
{
	if (argc == 2) {
		if (strcmp(argv[1], "stop-beacon") == 0) {
			timer_.cancel();
			return TCL_OK;
		}
		else if (strcmp(argv[1], "start-beacon") == 0) {
			timer_.resched(Random::uniform(0, beacon_));
			return TCL_OK;
		}
      		else if (strcmp(argv[1], "dump") == 0) {
			dump();
			return TCL_OK;
		}
	}
	if (argc == 3) {
		if (strcmp(argv[1], "set-beacon-period") == 0) {
			beacon_ = atof(argv[2]);
			return TCL_OK;
		}
		if (strcmp(argv[1], "beacon-period") == 0) {
			beacon_ = atof(argv[2]);
			timer_.resched(Random::uniform(0, beacon_));
			return TCL_OK;
		}
		if (strcmp(argv[1], "ragent") == 0) {
		  	ragent_ = (NsObject *)TclObject::lookup(argv[2]);
			return TCL_OK;
		}
	}
	return (Agent::command(argc, argv));
}

// *************************************************************************
// Beacons are always broadcast packets
// XXX: should be done in the NETWORK Agent, not in MIP - just needs an API
// *************************************************************************
void BSAgent::send_ads()
{
   Packet *p = allocpkt();
   hdr_ip *iph = HDR_IP(p);
   hdr_rtads *rh = HDR_RTADS(p);
   hdr_cmn *hdrc = HDR_CMN(p);

   iph->daddr() = IP_BROADCAST;
   iph->dport() = port();
   hdrc->ptype() = PT_RADS;
   hdrc->size() = PT_RADS_SIZE;

   if (print_info_) 
	cout << "+ BS " <<  MYNUM << " send ADS to " 
	<< PRINTADDR(iph->daddr()) << " at " << NOW << "\n";

   rh->lifetime() = adlftm_;

   //if (bcast_target_) bcast_target_->recv(p, (Handler*) 0);
   //else if (target_) send(p, 0);
   //else Packet::free(p);
   send(p, 0);
}


// *************************************************************************
// >----------------------------------------------------------------------<
// Mobile IPv6 Mobile Node 
// If advertisement from a new BS: Get a new COA
// Periodically: Register current COA with HA, former BS, and CNs
// >----------------------------------------------------------------------<
// *************************************************************************

void BSListTimer::expire(Event *) {
	a_->timeout(MIP_TIMER_BSLIST);
}

static class MNAgentClass : public TclClass {
public:
	MNAgentClass() : TclClass("Agent/MN") {}
	TclObject* create(int, const char*const*) {
		return (new MNAgent());
	}
} class_mnagent;

// *************************************************************************
// MNAgent Creator
// *************************************************************************
MNAgent::MNAgent() : MIPv6Agent(), ha_(NONE), bs_(NONE), oldbs_(NONE),
	coa_(NONE), oldcoa_(NONE), coalost_(TRUE), nbu_(0),
	reglftm_(~0), adlftm_(0.0), node_ (0), bu_policy_(DEF_BU), 
	rt_opti_(1), bs_forwarding_(1), decap_port_(-1), 
	bslist_timer_(this), bu_timer_(this)
{	
	// Binding Update List Management 
        LIST_INIT(&bulist_head_);

	// List of Base Station
	LIST_INIT(&bslist_head_);
	
	// Binding Update History - to collect statistics
        LIST_INIT(&history_head_);

	bind("max_rate_", &max_rate_);
	bind("slow_rate_", &slow_rate_);
	bind("reg_lifetime_", &reglftm_);
	bind("rt_opti_", &rt_opti_);
	bind("bs_forwarding_", &bs_forwarding_);
	m_seqno_ = 0;
	strcpy(node_info_,"MN");
}

// *************************************************************************
// MNAgent Recv
// *************************************************************************
void MNAgent::recv(Packet* p, Handler *)
{
   hdr_cmn *hdrc = HDR_CMN(p);
   hdr_ip *iph = HDR_IP(p);

   if (print_info_) 
	cout << "- " << node_info_ << " "  <<  MYNUM << " at " << NOW 
	<< " from " << PRINTADDR(iph->saddr());
 
   if (iph->dport() == port() ) {
     // This packet is intended for MIPv6 specifically

     switch (hdrc->ptype()) {
      case PT_RADS:
	   if (print_info_) cout << " -- Router ADS";
	recv_ads(p);
	   if (print_info_) cout << "\n";
	Packet::free(p);
	break;

      case PT_BU:
	   if (print_info_) cout << " -- BU\n";
	// fprintf(stderr, "MIPv6 MN: processing BU from another MN not yet implemented");
	cout << NOW << " MIPv6 MN: processing BU from another MN not yet implemented\n";

//	exit(1);
	// recv_bu(p);
	Packet::free(p);
	break;

      case PT_BACK:
	   if (print_info_) cout << " -- BACK\n";
	recv_back(p);
	Packet::free(p);
	break;

      case PT_BREQ:
	   if (print_info_) cout << " -- BREQ\n";
	// recv_breq(p);
	fprintf(stderr, "MIPv6: BINDING REQUEST not yet implemented");
	exit(1);
	Packet::free(p);
	break;

      case PT_SOL:
           // Recv Solicitation from an other MN - discard   
           Packet::free(p);
           break;

      default:
	fprintf(stderr, "MIPv6: Packet Type %d not supported\n", hdrc->ptype());
	Packet::free(p);
	break;
     } // switch

   } else {
        // XXX Once we add piggybacking, this should be processed here.
        // Keep forwarding this packet
	// send(p, 0);
	fprintf(stderr, "Piggybacking not yet supported\n");
	Packet::free(p);
   }
}

// *************************************************************************
// If we received an encapsulated packet, we probably need to send a BU 
// to sender because it doesn't have the careof-address.
// *************************************************************************
void MNAgent::recv_decap(Packet *p)
{
/*
   hdr_ip *iph = HDR_IP(p);
   if ( iph->daddr() == here_.addr_ ) {
	add_bulist( iph->saddr(), BU_CN, ON);
   }
*/
}

// *************************************************************************
// Processing of Binding Acknowledgment    
// Stop sending BUs if we are back to home and ACK from HA for the right CoA
// XXX: should do it on a per CN or HA basis, not for everyone.  
// *************************************************************************
void MNAgent::recv_back(Packet *p)
{ 
   hdr_mipv6 *miph = HDR_MIPv6(p);
   hdr_ip *iph = HDR_IP(p);

   Entry *n = lookup_entry(iph->saddr(), head(bulist_head_));
   if ( n != NULL ) {
      if ( n->seqno == miph->seqno() && n->caddr == coa_ && iph->saddr() == ha_ && coa_ == HADDR) { 
	if (bu_timer_.status() == TIMER_PENDING) {
		bu_timer_.cancel();
	}
      }
   }
}

// *************************************************************************
// Processing of Router Advertisement
// *************************************************************************
void MNAgent::recv_ads(Packet *p)
{
   hdr_ip *iph = HDR_IP(p);
   hdr_rtads *rh = HDR_RTADS(p);

   // We search in list of BSs to check if we know about this BS
   Entry *node = lookup_entry(iph->saddr(), head(bslist_head_));
   if ( node ) { 
      // We already have this BS in the list - Reset its expire time 
      // No need to send BU 

      node->update_entry(NONE, iph->saddr(), NONE, NOW, rh->lifetime());

   } else { 
      // New ads.  This BS is not yet recorded in our BS list
      // Record it in front of list. This BS will become current BS
      // We need a new COA on the BS's subnet and we need 
      // to register BS in list and new binding with HA / CN.
      
      if (print_info_) cout << " ***  new BS ***"; 
      Entry *pbs = add_bs(iph->saddr());

      // XXX: I should send a request to obtain a COA 
      pbs->update_entry(NONE, NONE, get_coa(iph->saddr()), NOW, rh->lifetime());
      coalost_ = TRUE;
      reg();
   }
}

// *************************************************************************
// Timer
// *************************************************************************
void MNAgent::timeout(int tno)
{
   switch (tno) {
   case MIP_TIMER_BU:
	// Happens if no BU sent during max_rate_ or slow_rate interval
	if (print_info_) 
	   cout << "* MN " << MYNUM << " Timer BU " 
		<< max_rate_ << " expired at " << NOW << "\n";
	reg();
	break;

   case MIP_TIMER_BSLIST: {
	// Check expiration time for each BS
	// If expired, delete it since no ads for too long
	// If this is the current BS, register with a new one.

	if (print_info_) 
	cout << "* MN " << MYNUM << " Timer BSList expired at " << NOW << "\n";

	// Remove entries that have expired
	Entry *entry = head(bslist_head_); 
   	for ( ; entry; entry = entry->next_entry()) {
	   if (entry->expire() < NOW) {
	   	if ( entry->caddr == coa_) {
		   // We have lost the current BS
		   // We will need to register with a new one 
		   coalost_ = TRUE;
		}
		entry->remove_entry();
		delete entry;
	   }
	}
	bslist_timer_.resched(beacon_);
	
	if (coalost_ == TRUE) {
	   if (print_info_) cout << "* MN lost contact with current BS" << "\n";
	   reg();
	}
	break;
   }
   default:
	break;
   }
}

// *************************************************************************
// Interface to the OTcl interpreter
// *************************************************************************
int MNAgent::command(int argc, const char*const* argv)
{
   if (argc == 2) {
      if (strcmp(argv[1], "dump") == 0) {
	dump();
	return TCL_OK;
      }
   } else if (argc == 3) {
      if (strcmp(argv[1], "check-beacon") == 0) {
	// We check if we are still attached to the BS every <beacon_> sec.
	beacon_ = atof(argv[2]);
	timeout(MIP_TIMER_BSLIST);
	bslist_timer_.resched(beacon_);
	return TCL_OK;
      }
      else if (strcmp(argv[1], "set_max_rate") == 0) {
	max_rate_ = atof(argv[2]);
	bu_timer_.resched(max_rate_);
	return TCL_OK;
      }
      else if (strcmp(argv[1], "set_slow_rate") == 0) {
	slow_rate_ = atof(argv[2]);
	return TCL_OK;
      } else if (strcmp(argv[1], "decap-port") == 0) {
	decap_port_ = atoi(argv[2]);
	return TCL_OK;
      } else if (strcmp (argv[1], "node") == 0) {
	node_ = (MobileNode*)TclObject::lookup(argv[2]);
	if (node_ == 0) {
	   fprintf (stderr, "%s: %s lookup of %s failed\n", __FILE__, argv[1], argv[2]);
	   return TCL_ERROR;
	}
	return TCL_OK;
      }
      else if (strcmp (argv[1], "add-cn") == 0) {
	   Entry *nn = add_bulist(atoi(argv[2]), BU_CN, ON);
           nn->activate_entry(NOW, TIME_INFINITY);
	   return TCL_OK;
      }
      else if (strcmp (argv[1], "remove-cn") == 0) {
	   remove_bulist(atoi(argv[2]), BU_CN );
	   return TCL_OK;
      }
      else if (strcmp (argv[1], "add-group") == 0) {
	   Entry *nn = add_bulist(atoi(argv[2]), BU_RP, ON );
           nn->activate_entry(NOW, TIME_INFINITY);
	   return TCL_OK;
      }
      else if (strcmp(argv[1], "set-ha") == 0) {
	 Entry *nn = add_bulist(ha_=atoi(argv[2]), BU_HA, ON);
         nn->activate_entry(NOW, TIME_INFINITY);
	 return TCL_OK;
      }
   }
   return (Agent::command(argc, argv));
}

// *************************************************************************
// Add a new node in the Binding Update List
// If node already exists, just activate the entry 
// Node = CN, HA, former BS
// *************************************************************************
Entry* MNAgent::add_bulist(int daddr, Mipv6RegType type, int flag)
{
   // Check that an entry does not already exists
   Entry *n = lookup_entry(daddr, head(bulist_head_));
   if ( n == NULL ) {
        n = new Entry(daddr, type, flag);
        n->insert_entry(&bulist_head_);
   }
   // In all case, activate the entry and allow BU to be sent until time limit
   // By default, registration time. 
   n->activate_entry(NOW, reglftm_); 
   return n;
}

// *************************************************************************
// Remove a node (CN, HA, BS) from the Binding Update List 
// Indeed, it is moved to the history list for statistics.
// Could also use	nn->remove_entry(&history_head_);
// *************************************************************************
void MNAgent::remove_bulist(int addr, Mipv6RegType type)
{
   Entry *node = lookup_entry(addr, head(bulist_head_));
   if (node && node->type == type) {
	// Move entry from the Binding List to history
	node->remove_entry();

        // Chain entry to the history if there exists one (for statistics)
   	node->insert_entry(&history_head_);
   }
}

// *************************************************************************
// Add new BS in Base Station List 
// *************************************************************************
Entry* MNAgent::add_bs(int daddr)
{
   // Check if an entry does not already exist
   Entry *node = lookup_entry(daddr, head(bslist_head_));
   
   assert (node == NULL);

   node = new Entry(daddr, BS_ADS, OFF);
   node->insert_entry(&bslist_head_);
   return node;
}

// *************************************************************************
// Print: 
// Binding Update List 
// Base Station List
// History List
// *************************************************************************
void MNAgent::dump()
{
   dump_list(head(bulist_head_), "Binding Update List");
   dump_list(head(bslist_head_), "Base Station List");
   dump_list(head(history_head_), "History List");
}

// *************************************************************************
// Get a new coa on the BS's subnet
// We need a new COA with same prefix as the new current BS
// XXX: for now, call TCL interpreter, but we need something like DHCPv6
// XXX: Interpreter returns a COA with BS's prefix
// *************************************************************************
int MNAgent::get_coa(int new_bs)
{
   Tcl& tcl = Tcl::instance();

   if ( new_bs == ha_ )
	// We are back to home.  
	// We still need to (de-)register with coa = home_address
	return HADDR;
   else {
cout << NOW << " get_coa for BS " << PRINTADDR(new_bs) << ":" << new_bs << "\n";
	tcl.evalf("%s get-coa %s", name_, PRINTADDR(new_bs) );
	return(atoi(tcl.result()));
   }
}

// *************************************************************************
// We are now attached to a new subnet 
// The MNagent now should update the Mobilenode about the changed coa_ 
// 1. ARP needs to reply to this COA
// 2. Current BS is needed by Network Agent to set "next_hop"
// *************************************************************************
void MNAgent::set_subnet()
{
	node_->set_coa(coa_);
	node_->set_base_stn(bs_);
	if (print_info_) {
		cout << " MN " << node_->address();
		cout << " has COA " << PRINTADDR(node_->coa()) << "\n"; 
	}
}      

// *************************************************************************
// Registration with HA, CNs, previous default router
// When ?
// => New BS
// => Timer has expired
// => BU Request (XXX: not yet implemented)
// Send current COA to:
// - HA: If we are back home, COA would be = home address. Fine. 
// - Previous BS: unless previous BS is HA or we don't have one 
// 	XXX: How long should we keep forwarding at previous BS ?
// - CNs if we have some in the Binding List
// [MIPv6] says that BUs are generally sent to CNs 
// when a BU containing a new COA is sent to the HA.
// Here, we send BUs to CNS whenever we send a BU to HA. 
// *************************************************************************
void MNAgent::reg()
{
   // Check if we are still attached to a BS
   if ( ! head(bslist_head_) ) {
   	// We do not have BS at all in the BS list 
	bu_timer_.resched(max_rate_);
	send_sols();
	return;
   }
   
   // Check if we have an active care-of address
   if (coalost_ == TRUE) {
	// We have BSs in the list, but we have lost contact with current BS
	// because TIMER_BSLIST has expired. 
	// Choose new BS = first on list since we have deleted the former one 
	// and register with it                         

        // Stop sending BUs at "previous previous" BS if still in BU List 
	// XXX: this should be automatic - we don't keep it very long (?)
        remove_bulist(oldbs_, BU_BS);

	Entry *pbs = head(bslist_head_); 
	oldbs_ = bs_;
	bs_ = pbs->addr;
	oldcoa_ = coa_; 
	coa_ = pbs->caddr;
	adlftm_ = pbs->lifetime();
      
	// Set the new BS as the gateway to the Internet
	set_subnet();
	coalost_ = FALSE;
	nbu_ = 0; 

	// We need to establish forwarding from previous BS
	// if there is one
	// NOTE: if previous BS is HA, add_bulist already have
	// an entry and won't add a new one
	// XXX: if ( oldbs_ != NONE && oldbs_ != ha_ ) {
	if ( oldbs_ != NONE ) {
		Entry *nn = add_bulist(oldbs_, BU_BS, ON);
		nn->update_entry(NONE, oldcoa_, coa_, NOW, reglftm_);
	}
   }

   // As specified in Mobile IPv6, periodic BU should be sent 
   // with longer interval after 5 transmissions
   if ( nbu_++ < MAX_FAST_UPDATES ) 
	bu_timer_.resched(max_rate_);
   else
	bu_timer_.resched(slow_rate_);

   send_standard_bu(BU_HA);
   if ( rt_opti_ ) send_standard_bu(BU_CN);
   if ( bs_forwarding_) send_standard_bu(BU_BS);
}

// *************************************************************************
// Send standard Mobile IPv6 Binding Update  
// XXX: Here, we may decide to send BUs to CNs and HAs at 
// XXX: different frequency, depending on local policy
// XXX: For now, send BU to everyone at the same time
// *************************************************************************
void MNAgent::send_standard_bu(Mipv6RegType type) 
{
   Entry *n = head(bulist_head_); 
   Entry *nn;
   Packet *p;
   hdr_mipv6 *h;
   while ( n ) { 
      nn=n;	
      n=n->next_entry();
      if ( nn->activated(NOW) && (nn->type == type) ) {
	p = set_bu_packet(type, nn->addr, coa_);
	h = HDR_MIPv6(p);

	// Update Binding Update List
	nn->update_entry(h->seqno(), h->haddr(), h->coa(), NOW, h->lifetime());
	send_packet(p);
      }
   }
}

// *************************************************************************
// Fill Binding Update = IPv6 header + Binding Update Option header
// *************************************************************************
Packet* MNAgent::set_bu_packet(Mipv6RegType type, int dst, int newad)
{
        Packet *p = allocpkt();
	set_ipv6_hdr(p, dst);	
	set_bu_hdr(p, type, newad);	
	return(p);
}

// *************************************************************************
// Fill the Binding Update Option Header 
// *************************************************************************
void MNAgent::set_bu_hdr(Packet *p, Mipv6RegType type, int newad) 
{
	hdr_cmn *hdrc = HDR_CMN(p);
        hdr_mipv6 *h = HDR_MIPv6(p);
	
	// By default, set flags to OFF.
	h->H() = OFF;
	h->A() = OFF;
	
	if ( type != BU_BS ) {
        	h->haddr() = HADDR; 
	}
	else {
        // Former BS establish forwarding from previous coa to new coa
	// XXX: we should better take the address recorded in BU List
		h->haddr() = oldcoa_;
	}

	if ( type == BU_HA ) {
		h->H() = ON;
		h->A() = ON;
	}
        h->coa() = newad;
        h->type() = BU;
        h->lifetime() = LIFETIME;
	h->seqno() = ++m_seqno_;
   
	hdrc->size() += BU_SIZE;
	hdrc->ptype() = PT_BU;
}

// *************************************************************************
// When the MN does not have a registered BS, it needs to enquire for one
// XXX: should be done in the NETWORK Agent, not in MIP - just needs an API
// *************************************************************************
void MNAgent::send_sols()
{
   Packet *p = allocpkt();
   hdr_ip *iph = HDR_IP(p);
   hdr_mipv6 *h = HDR_MIPv6(p);
   hdr_cmn *hdrc = HDR_CMN(p);

   h->haddr() = HADDR;
   hdrc->ptype() = PT_SOL;
   iph->daddr() = IP_BROADCAST;
   iph->dport() = port();
   hdrc->size() = PT_RSOL_SIZE;

   if (print_info_) cout << "+ MN send SOLICITATION at " << NOW << "\n";

   if (bcast_target_) bcast_target_->recv(p, (Handler*) 0);
   else if (target_) send(p, 0);
   else Packet::free(p);
}

// *************************************************************************
// >----------------------------------------------------------------------<
// Mobile IPv6 Correspondent Node
// >----------------------------------------------------------------------<
// *************************************************************************
static class CNAgentClass : public TclClass {
public:
        CNAgentClass() : TclClass("Agent/CN") {}
        TclObject* create(int, const char*const*) {
                return (new CNAgent());
        }
} class_cnagent;

// *************************************************************************
CNAgent::CNAgent() : MIPv6Agent(), timer_(this) 
{
   strcpy(node_info_,"CN");
}

// *************************************************************************
void CNAgent::timeout(int tno)
{
//      timer_.resched(lifetime);
//      send_request();
}

// *************************************************************************
int CNAgent::command(int argc, const char*const* argv)
{
  if (argc == 2) {
      if (strcmp(argv[1], "dump") == 0) {
        dump();
        return TCL_OK;
      }
   }
   return (Agent::command(argc, argv));
}

// *************************************************************************
// Print:
// Binding Cache
// *************************************************************************
void CNAgent::dump()
{
   dump_list(head(bcache_head_), "Binding Cache");
}
